// calcul sin cos par cordic.
// implantation dans AVR avec coeur cordic en VHDL
// balayage de l'angle depuis -pi/2 jusque +pi/2
// renvoie du resultat vers partie VHDL de gestion d'ecran VGA

// NB !!! la vitesse de rotation du point peut se modifier pour
// 	un ralentissement :
//	- soit en appliquant un délai plus important entre deux calculs de position du point ( _delay_ms( ); )
//	- soit en reduisant la variation d'angle entre deux calculs de position du point ( beta_q3+=0x0010  )
//	- soit en combinant les deux effets précédents

// pour augmenter la vitesse de rotation le même principe avec modification des paramètres dans l'autre sens !
// empiriquement, un délai nul donne un affichage moche . il faut au moins 1 ms d'attente 



// amelioration / cordic_avr4 : ajout de la verification de la chaine de caractere
// correspondant a l'angle envoye par GTKTerm ( format +x.xxxx ou -x.xxxx)

#include "avr/io.h" 
#undef F_CPU 
#define F_CPU 50000000UL 
#include "util/delay.h" 



#include <stdio.h>
#include <string.h>

#include "crc_attiny.h"


#define UCSRB	_SFR_IO8(0x01) 
#define UCSRA	_SFR_IO8(0x02) 
#define UDR	_SFR_IO8(0x03)
// UCSRA
#define RXC 7
#define TXC 6
#define UDRE 5
//UCSRB
#define RXEN 4
#define TXEN 3



//************************************************************************
// function uart_init()
// purpose: put character in first rs232 PORT
// arguments:
// corresponding character
// return:
// note: 38400,8,n,2 hard coded : transmission only
//************************************************************************
void usart_init(void) {
UCSRB = (1<<TXEN); // transmission
// UCSRC = (1<<UCSZ1)|(1<<UCSZ0)|(1<<URSEL); // ignoré par notre matériel
// UBBRL = 0x33; //pas pour nous car 38400 bauds Sans 8,2
}
//************************************************************************
// function uart_send()
// purpose: put character in first rs232 PORT
// arguments:
// corresponding character
// return:
// note: 38400,8,n,2 hard coded
//************************************************************************
void usart_send(unsigned char ch){
while(!(UCSRA & (1<<UDRE)));
UDR = ch;
}

//************************************************************************
// function uart_receive()
// purpose: read character in second rs232 PORT
// arguments:
// corresponding character
// return: non-blocking sub return 1 if no data present else return char
// note: 38400,8,n,2 hard coded, non-blocking sub return 0 if no data present
//************************************************************************
char usart_receive(void){
if (UCSRA & (1<<RXC)) //attente tant que Data Present en réception
return UDR;
else return 0;
}

// function usart_gets()
// purpose: recupere l'angle a traiter par cordic au format
// +/- x.xxxx en decimal depuis un envoi par GTKTerm via la liaison serie RS232
// teste la validite de la chaine entree et demande de recommencer s'il y a erreur de format :
// teste le signe + ou-
// test la virgule
// test si les chiffres (0 -9) en sont bien 
void usart_gets(char str[])
{
	uint8_t i=0;
	uint8_t j=0;
	uint8_t test_str=1; // flag de validite
	char str_msg5[]={"\n\nSaisie de l'angle incorrecte ! \n Veuillez la recommencer dans le format +x.xxxx ou -x.xxxx\n\n"};
	char car;
	while(i!=7)	
	{
		car=usart_receive();  // saisie de 7 caracteres
		if (car != 0) {
			str[i]=car;
			i++;
		}
		if (i==7) {	// teste la chaine en fin de saisie
			test_str=1; // a priori ok
			if (str[0]=='+')	//test le signe
				{test_str=1;}
			else if (str[0]=='-')
				{test_str=1;}
			else	{test_str=0;}
			if (str[2]!= '.')	// test la virgule
				{test_str=0;}
			if (str[1]<'0' || str[1]>'9')	// test la partie entiere
				{test_str=0;}
			for (j=3;j<7;j++) {		// teste la partie decimale
				if (str[j]<'0' || str[j]>'9')
					{test_str=0;}
			}
			if (test_str==0) {	// verifie en fin de test si la chaine est ok
				i=0;
				while(str_msg5[i]!=0) { // demande la re-saisie de la chaine si necessaire
					usart_send(str_msg5[i]);
					i++;
				}
				i=0; // reinitialisation de l'index de saisie des 7 caracteres
			}
		}
				

	}
}


// conversion chaine de caractere en q3.13
// initialisation uart prealable requise
int StringToQ3_13(char str[]){
	uint8_t i=0;
	int var=0;
	int valq3_13=0;
	valq3_13=(str[1]-'0')<<13; // partie entiere dans le 2ieme bit
	var=(str[3]-'0')*1000 +(str[4]-'0')*100 +(str[5]-'0')*10 +(str[6]-'0'); // partie decimale sous forme d'un entier ( 0,5312 => 5312)
	for(i=0;i<13;i++) {	// comparaison avec les puissances de -i  (0,5 0,25 0,125 0,0625 ...) soit en entier 5000 2500 1250 625 ...
		if (var>=(5000>>i)) {    //  remplissage des bit correspondants a 0 ou 1 selon comparaison et soustraction le cas échéant du poids rencontré 
			valq3_13 = valq3_13 | (1<<(12-i));
			var=var-(5000>>i);
			}
		}
	if (str[0]=='+')	// test le signe pour envoi du nombre ou de son opposé
	return valq3_13;
	else
	return -valq3_13;
}


// conversion du q3.13 en chaine de caractere
void HexQ3_13ToString(int valQ3_13,char str[]){
unsigned int valQ3_13b;
char digit;
	if (valQ3_13 < 0) { // eviter problèmes de signe !!!
		str[0] = '-';
		valQ3_13 = -valQ3_13;
	} else
		str[0] = '+';
		digit = valQ3_13 >> 13;
		str[1]= digit + '0';
		str[2]= '.';
		valQ3_13 &= 0x1FFF; // on retire les 3 bits de poids fort
		valQ3_13 = valQ3_13 * 5; //*5 pour tenir dans 16 bits
		valQ3_13b = valQ3_13;
		valQ3_13b >>= 12; // on ne garde que les 4 bits de poids fort
		str[3] = valQ3_13b +'0';
		valQ3_13 &= 0x0FFF; // on retire les 4 bits de poids fort
		valQ3_13 = valQ3_13 * 10; //enfin le * 10
		valQ3_13b = valQ3_13;
		valQ3_13b >>= 12; // on ne garde que les 4 bits de poids fort
		str[4] = valQ3_13b +'0';
		valQ3_13 &= 0x0FFF; // on retire les 4 bits de poids fort
		valQ3_13 = valQ3_13 * 10; //enfin le * 10
		valQ3_13b = valQ3_13;
		valQ3_13b >>= 12; // on ne garde que les 4 bits de poids fort
		str[5] = valQ3_13b +'0';
		valQ3_13 &= 0x0FFF; // on retire les 4 bits de poids fort
		valQ3_13 = valQ3_13 * 10; //enfin le * 10
		valQ3_13b = valQ3_13;
		valQ3_13b >>= 12; // on ne garde que les 4 bits de poids fort
		str[6] = valQ3_13b +'0';
		str[7]=0;  // caractere de fin de chaine : a ne pas afficher
	}



int main(void)
{
	
	int8_t i=0;
	// table des alphai = atan(2 puissance -i) pour i = 0 à 13
//	int16_t alphai_q3[]={0x1921,0x0ED6,0x07D6,0x03FA,0x01FF,0x00FF,0x007F,0x003F,0x001F,0x000F,0x0007,0x0003,0x0002,0x0001};
	char str[10]; 
//	char str_msg3[]="Entrez l'angle à traiter en decimal\n";
//	char str_msg4[]="format 7 caracteres +/-;x;.;x;x;x;x\n";
//	char str_msg5[]="angle (radians) : ";
	char str_msg6[]="cosinus : ";
	char str_msg7[]="sinus   : ";	
//	char str_msg8[]="Calcul cordic en cours ... ";
//	char str_msg9[]="Calcul cordic terminé, lecture résultat ...\n ";	
//	float k=0.60725294;
//	int16_t k=0x136E; // coeff k au format q3.13

//	beta : Angle à traiter (cos et sin) au format q3.13
//	int16_t beta_q3;
// 	variable intermediaire et resultat du cosinus et du sinus
	int16_t x,y;
	int8_t done=0;
	int16_t alpha;

	usart_init(); // pour pouvoir déclencher interruption RS232
	
	TCNT1 = 0x08; // initialisation de la fsm du coeur cordic matériel
	_delay_ms(100);
	TCNT1 = 0x00; // arret de l'initialisation de la fsm


while(1){
	

//	presque -pi/2 à +pi/2
//	for(beta_q3=0xCDBF;beta_q3<0x3241;beta_q3+=0x0020)

		

//	presque -pi a pi
//	for(beta_q3=0x9B79;beta_q3<0x6487;beta_q3+=0x001A)
//	{

// angle en degré (entier signé 16 bits )
	for (alpha=-179;alpha<180;alpha++)
	{
	//beta_q3 = 0x2000;
		// envoi de l'angle vers coeur cordic materiel
//		PORTB = (uint8_t)(beta_q3 & 0x00FF);
//		PORTA = (uint8_t)((beta_q3>>8) & 0x00FF);

		// envoi de l'angle vers coeur cordic materiel
		PORTB = (uint8_t)(alpha & 0x00FF);
		PORTA = (uint8_t)((alpha>>8) & 0x00FF);


		done = 0x00;	// initialisation du flag de
		// envoi de la commande de demarrage de calcul : startcordic 
		TCNT1 = 0x80;

		// attente de la fin du calcul
		while(done != 0x01) {
		  /*i=0;
		  while(str_msg8[i]!=0) {
		  usart_send(str_msg8[i]);
		  i++;
		  }*/
		done = TCCR1B & 0x01 ; // lecture du flag de fin de calcul : cordicdone
		  }
/*	
		//lecture resultat
		x = PINA;	
		x = (x<<8) + PINB;
		y = ADCH;
		y = (y<<8) + ADCL;
*/
		    /*i=0;
		    while(str_msg9[i]!=0) {
		    usart_send(str_msg9[i]);
		    i++;
		  }*/
		//_delay_ms(2);

		// acquittement envoyé au coeur cordic NB !! necessaire pour laisser la machine d'etat retourner dans son etat initial d'attente
		TCNT1 = 0x00;
		// pour la prochaine boucle d'attente de fin de calcul
		done = 0x00;
	

		// conversion resultat cosinus en chaine de caractere 
		HexQ3_13ToString(x,str);
		// envoi resultat cosinus
		i=0;
		while(str_msg6[i]!=0) {
		usart_send(str_msg6[i]);
		i++;
		}
		for (i=0;i<7;i++)  // on arrete a 6 pour ne pas afficher le caractere de fin de chaine
		usart_send(str[i]);
		usart_send(0x0A);
		usart_send(0x0D);
		// conversion resultat sinus en chaine de caractere 
		HexQ3_13ToString(y,str);
		i=0;
		while(str_msg7[i]!=0) {
		usart_send(str_msg7[i]);
		i++;
		}
		// envoi resultat sinus
		for (i=0;i<7;i++)
		usart_send(str[i]);
		// retour chariot
		usart_send(0x0A);
		usart_send(0x0D);

	

	}

}

}